import { h, Component, cloneElement } from 'preact'
import 'whatwg-fetch'

class Image extends Component {
	state = {
		blob: null,
		loading: true,
		delayed: false,
		thumbnail: null
	}

	setBlob = blob => this.setState({ blob })

	setThumbnail = thumbnail => this.setState({ thumbnail })

	setLoading = loading => this.setState({ loading })

	fetchImage = src => {
		fetch(src)
			.then(res => res.blob())
			.then(res => {
				this.setBlob(URL.createObjectURL(res))
				this.setLoading(false)
			})
			.catch(err => console.log(err))
	}

	delayFetchingImage = src => this.setState({ delayed: true })

	dataSaverMode = src => this.delayFetchingImage(src)

	image = () => this.imgElement

	scrollToReveal = src => {
		try {
			let observer = new IntersectionObserver(entries => {
				let image = entries[0]

				if (image.isIntersecting) {
					this.fetchImage(src)
					this.setState({ delayed: false })
					observer.disconnect()
				}
			})

			observer.observe(this.image())
		} catch (_) {
			// Fail gracefully
			console.warn('ScrollToView not supported on this browser')
			this.fetchImage(src)
			this.setState({ delayed: false })
		}
	}

	componentDidMount() {
		const { children, dataSaver, src, scrollToReveal, thumbnail } = this.props

		// No Child ? Assume it is a cloudinary image and fetch
		// Note: VNode is always an array in preact
		if (!children.length) {
			let thumbnail =
				thumbnail || src.replace('/upload/', '/upload/c_thumb,w_30/')

			if (dataSaver || (dataSaver && scrollToReveal)) {
				this.dataSaverMode(src)
			} else if (!dataSaver && scrollToReveal) {
				this.scrollToReveal(src)
			} else {
				this.fetchImage(src)
			}

			this.setThumbnail(thumbnail)
		}
	}

	shouldRenderButton(dataSaver) {
		if (!dataSaver) return

		return (
			<button
				className="pimg__btn"
				onClick={dataSaver ? () => this.fetchImage(this.props.src) : null}
			>
				Load image
			</button>
		)
	}

	render() {
		const {
			children,
			className,
			dataSaver,
			loadingClassName,
			scrollToReveal,
			src,
			thumbnail,
			...rest
		} = this.props

		const { blob, loading } = this.state

		// No Child ? Just Render as usual
		if (!children.length) {
			if (loading) {
				const classes = className
					? `${className} ${loadingClassName || className + '__loading'}`
					: 'pimg pimg__loading'
				return (
					<div className="pimg__wrapper">
						<img
							className={classes}
							src={thumbnail || this.state.thumbnail}
							ref={i => (this.imgElement = i)}
							{...rest}
						/>
						{this.shouldRenderButton(dataSaver)}
					</div>
				)
			}

			return (
				<div className="pimg__wrapper">
					<img
						className={className ? className : 'pimg'}
						src={blob}
						{...rest}
					/>
				</div>
			)
		}

		// If a child is found, clone and send some prop
		return cloneElement(children[0], {
			blob,
			fetchImage: this.fetchImage,
			image: src,
			imageClassName: className,
			loading,
			setThumbnail: this.setThumbnail,
			thumbnail
		})
	}
}

class Thumbnail extends Component {
	componentDidMount() {
		const { fetchImage, image, setThumbnail, src } = this.props

		setThumbnail(src)

		fetchImage(image)
	}

	render() {
		const { blob, className, imageClassName, loading, thumbnail } = this.props

		if (loading) {
			return (
				<img
					className={
						className && imageClassName
							? `${className} ${imageClassName}`
							: 'pimg pimg_loading'
					}
					src={thumbnail}
				/>
			)
		}

		return (
			<img className={imageClassName ? imageClassName : 'pimg'} src={blob} />
		)
	}
}

Image.defaultProps = {
	dataSaver: false,
	src: ''
}

export default Image
export { Thumbnail }
